@@ -27,7 +27,7 @@ from .overview import Overview, OverviewState, PreviewState
 # TODO builtin frame/stack explorer? from objexplore import stackexplore
 
 
-version = "1.5.3"
+version = "1.5.4"
 console = Console()
 EDITOR = os.environ.get("EDITOR")
 
@@ -66,11 +66,7 @@ class ObjExploreApp:
                 try:
                     self.draw()
                     key = self.term.inkey()
-                    res = self.process_key_event(key)
-
-                    # If the object is returned as a response then close the explorer and return the selected object
-                    if res is not None:
-                        break
+                    self.process_key_event(key)
 
                 except RuntimeError as err:
                     # Some kind of error during resizing events. Ignore and continue
@@ -82,6 +78,9 @@ class ObjExploreApp:
                     # Otherwise it is a new error. Raise
                     else:
                         raise err
+                except StopIteration:
+                    res = self.explorer.selected_object
+                    break
 
         # Unhide the cursor
         print("\x1b[?25h", end="")
@@ -116,7 +115,7 @@ class ObjExploreApp:
             return
 
         if key in ("q", "Q", "r"):
-            return self.explorer.selected_object.obj
+            raise StopIteration
 
         # Help page ###########################################################
 
@@ -330,9 +329,12 @@ class ObjExploreApp:
             elif self.overview.preview_state == PreviewState.repr:
                 printable = self.explorer.selected_object.obj
 
-            elif self.overview.preview_state == PreviewState.source:
+            elif self.overview.preview_state == PreviewState.source and self.explorer.selected_object._source:
                 printable = self.explorer.selected_object.get_source(fullscreen=True)
 
+            else:
+                printable = self.explorer.selected_object.obj
+
             with console.capture() as capture:
                 console.print(printable)
 
